package com.restlet.client.log;

import com.restlet.client.log.impl.MockLogService;

import java.util.logging.Level;

import static java.util.logging.Level.FINE;
import static java.util.logging.Level.FINER;
import static java.util.logging.Level.INFO;
import static java.util.logging.Level.SEVERE;
import static java.util.logging.Level.WARNING;

public interface LogService {

    void debug(Class<?> clazz, String message, Object... arguments);

    void debug(Class<?> clazz, String message, Throwable e, Object... arguments);

    void fine(Class<?> clazz, String message, Object... arguments);

    void error(String message, Object... arguments);

    void error(String message, Throwable e, Object... arguments);

    void info(String message, Object... arguments);

    void warning(String message, Object... arguments);

    void log(Severity severity, String message, Object... arguments);

    class LogServiceHolder {
        public static LogService LOG_SERVICE = new MockLogService(LogServiceHolder.class);
    }

    enum Severity {
        Error(SEVERE) {
            @Override
            public void log(final LogService logService, final String message, final Class<?> clazz) {
                logService.error(message);
            }
        },
        Info(INFO) {
            @Override
            public void log(final LogService logService, final String message, final Class<?> clazz) {
                logService.info(message);
            }
        },
        Warning(WARNING) {
            @Override
            public void log(final LogService logService, final String message, final Class<?> clazz) {
                logService.warning(message);
            }
        },
        Debug(FINE) {
            @Override
            public void log(final LogService logService, final String message, final Class<?> clazz) {
                logService.debug(clazz, message);
            }
        },
        Fine(FINER) {
            @Override
            public void log(final LogService logService, final String message, final Class<?> clazz) {
                logService.fine(clazz, message);
            }
        };

        public final Level level;

        Severity(final Level level) {
            this.level = level;
        }

        public abstract void log(LogService logService, String message, Class<?> clazz);
    }
}
