package com.restlet.client.net.uri.impl;

import com.restlet.client.function.Predicate;
import com.restlet.client.net.http.request.HttpVersion;
import com.restlet.client.net.uri.UriSchemeTo;
import com.restlet.client.utils.Objects;
import com.restlet.client.utils.Sequence;
import com.restlet.client.utils.StringUtils;

import java.io.Serializable;
import java.util.List;

import static java.util.Arrays.asList;

public class UriSchemeToImpl implements UriSchemeTo, Serializable {

    public static final UriSchemeTo HTTPS_SCHEME = new UriSchemeToImpl("https", HttpVersion.V11, true);
    public static final UriSchemeTo HTTP_SCHEME = new UriSchemeToImpl("http", HttpVersion.V11, false);
    public static final UriSchemeTo DEFAULT_SCHEME = HTTPS_SCHEME;

    private static final List<UriSchemeTo> SUPPORTED_SCHEME = asList(HTTP_SCHEME, HTTPS_SCHEME);

    private static final long serialVersionUID = 5786500393808068720L;

    private boolean secure;
    private String version;
    private String name;

    public UriSchemeToImpl() {
        this(null, null, false);
    }

    public UriSchemeToImpl(final String name, final HttpVersion version, final boolean secure) {
        this.name = name;
        this.secure = secure;
        this.version = version == null ? null : version.name();
    }

    public static UriSchemeTo valueOf(final String value) {
        if (StringUtils.isBlank(value)) {
            return null;
        }
        if ("http".equalsIgnoreCase(value)) {
            return HTTP_SCHEME;
        }
        if ("https".equalsIgnoreCase(value)) {
            return HTTPS_SCHEME;
        }
        return null;
    }

    @Override
    public boolean isSecure() {
        return secure;
    }

    public void setSecure(final boolean secure) {
        this.secure = secure;
    }

    @Override
    public String getVersion() {
        return version;
    }

    public void setVersion(final String version) {
        this.version = version;
    }

    @Override
    public String getName() {
        return name;
    }

    public void setName(final String name) {
        this.name = name;
    }

    public static boolean isSupportedScheme(final String schemeName) {
        return Sequence.of(SUPPORTED_SCHEME)
                .some(new Predicate<UriSchemeTo>() {
                    @Override
                    public boolean test(final UriSchemeTo uriSchemeTo) {
                        return Objects.equals(valueOf(schemeName), uriSchemeTo);
                    }
                });
    }

    public static boolean isSupportedScheme(final UriSchemeTo scheme) {
        return scheme != null && isSupportedScheme(scheme.getName());
    }

}
