package com.restlet.client.script.impl;

import com.restlet.client.async.Promise;
import com.restlet.client.dao.EnvironmentsDao;
import com.restlet.client.dao.RepositoryDao;
import com.restlet.client.model.EntityTo;
import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.net.http.request.impl.EffectiveHttpRequestTo;
import com.restlet.client.script.ScriptService;

import java.util.Collections;

import static com.restlet.client.async.Promises.of;
import static com.restlet.client.utils.EntityCloner.ENTITY_CLONER;

public abstract class ExpressionRequestEvaluator {

    private final EnvironmentsDao environmentsDao;
    private final ScriptService scriptService;

    public ExpressionRequestEvaluator(final EnvironmentsDao environmentsDao, final ScriptService scriptService) {
        this.environmentsDao = environmentsDao;
        this.scriptService = scriptService;
    }

    protected abstract String encodeQueryString(String query);

    public Promise<EffectiveHttpRequestTo> effective(final HttpRequestTo requestTo, final RepositoryDao repositoryDao) {
        if (requestTo == null) {
            return of();
        }
        return getRequestEvaluator(repositoryDao).process(ENTITY_CLONER.clone(requestTo, Collections.<String, EntityTo>emptyMap()));
    }

    public Promise<Boolean> isDynamic(final EntityTo entity, final RepositoryDao repositoryDao) {
        return getRequestEvaluator(repositoryDao).isDynamic(entity);
    }

    private RequestProcessor getRequestEvaluator(final RepositoryDao repositoryDao) {
        return new RequestProcessor() {
            @Override
            protected String encodeQueryString(final String query) {
                return ExpressionRequestEvaluator.this.encodeQueryString(query);
            }

            @Override
            protected Promise<String> process(final String value) {
                return scriptService.evaluate(value, environmentsDao, repositoryDao);
            }
        };
    }
}
