package com.restlet.client.script.runtime.impl;

import com.restlet.client.function.Function;
import com.restlet.client.net.MessageHeaderTo;
import com.restlet.client.net.NamedValuedEntity;
import com.restlet.client.platform.json.JsonArray;
import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonObject;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.utils.Sequence;

import java.util.List;

import static com.restlet.client.utils.Objects.isNullOrEmpty;
import static com.restlet.client.utils.StringUtils.isBlank;

public class JsonReferenceUtils {

    static JsonObject buildJsonObjectForNamedValuedEntity(final JsonEngine jsonEngine, final List<NamedValuedEntity> namedValuedEntities) {
        final JsonObject jsonObject = jsonEngine.newJsonObject();
        for (final NamedValuedEntity namedValuedEntity : namedValuedEntities) {
            if (namedValuedEntity == null || isBlank(namedValuedEntity.getName()) || namedValuedEntity.getValue() == null) {
                continue;
            }

            final JsonValue jsonValue = jsonObject.get(namedValuedEntity.getName().toLowerCase());
            if (jsonValue == null) {
                jsonObject.put(namedValuedEntity.getName().toLowerCase(), jsonEngine.newJsonString(namedValuedEntity.getValue()));

            } else if (jsonValue.isArray() != null) {
                jsonValue.isArray().set(jsonValue.isArray().size(), jsonEngine.newJsonString(namedValuedEntity.getValue()));

            } else if (jsonValue.isString() != null) {
                final JsonArray jsonArray = jsonEngine.newJsonArray();
                jsonArray.set(0, jsonValue);
                jsonArray.set(1, jsonEngine.newJsonString(namedValuedEntity.getValue()));
                jsonObject.put(namedValuedEntity.getName().toLowerCase(), jsonArray);
            }
        }
        return jsonObject;
    }

    public static JsonObject getHeaders(final JsonEngine jsonEngine, final List<MessageHeaderTo> headers) {
        if (isNullOrEmpty(headers)) {
            return jsonEngine.newJsonObject();
        }

        final JsonObject queryObject = buildJsonObjectForNamedValuedEntity(jsonEngine, Sequence.of(headers).map(new Function<MessageHeaderTo, NamedValuedEntity>() {
            @Override
            public NamedValuedEntity apply(final MessageHeaderTo header) {
                return header;
            }
        }).toList());
        return queryObject;
    }
}
