package com.restlet.client.script.runtime.impl;

import com.restlet.client.platform.json.JsonEngine;
import com.restlet.client.platform.json.JsonObject;
import com.restlet.client.platform.json.JsonValue;
import com.restlet.client.utils.FunctionalUtils;

import static com.restlet.client.utils.FunctionalUtils.ifNotNull;
import static com.restlet.client.utils.StringUtils.isBlank;

public class ResponseJSONReference {

    public final JsonObject root;
    private final JsonEngine jsonEngine;

    public ResponseJSONReference(final JsonObject root, final JsonEngine jsonEngine) {
        this.root = root;
        this.jsonEngine = jsonEngine;
    }

    JsonValue asJSON() {
        return root;
    }

    public void setStatus(final Integer statusCode, final String statusMessage) {
        if (statusCode != null || !isBlank(statusMessage)) {
            final JsonObject status = jsonEngine.newJsonObject();
            if (statusCode != null) {
                status.put("code", jsonEngine.newJsonNumber(statusCode));
            }
            if (!isBlank(statusMessage)) {
                status.put("message", jsonEngine.newJsonString(statusMessage));
            }
            root.put("status", status);
        }
    }

    public void setBody(final String text) {
        if (!isBlank(text)) {
            try {
                final JsonValue json = jsonEngine.fromJson(text);
                root.put("body", ifNotNull(json, FunctionalUtils.<JsonValue>identity(), jsonEngine.newJsonString(text)));

            } catch (final Throwable e) {
                root.put("body", jsonEngine.newJsonString(text));
            }
        }
    }
}
