package com.restlet.client.tests;

import com.restlet.client.function.Consumer;
import com.restlet.client.model.EntityTo;

import java.util.List;
import java.util.Set;

import static com.restlet.client.tests.TestResult.State.Aborted;
import static com.restlet.client.tests.TestResult.State.Error;
import static com.restlet.client.tests.TestResult.State.Failure;
import static com.restlet.client.tests.TestResult.State.InProgress;
import static com.restlet.client.tests.TestResult.State.Ok;
import static com.restlet.client.tests.TestResult.State.Skipped;
import static java.util.Arrays.asList;

public interface TestResult<T extends TestResult, E extends EntityTo> {

    enum ContainerStateComputationPolicy {
        AT_LEAST_ONE_SUCCESS(Failure, asList(Ok, Aborted, Skipped)),
        IN_PROGRESS_OVER_FAILURE(Ok, asList(Aborted, InProgress, Failure, Error)),
        STOP_ON_FAILURE(Ok, asList(Aborted, Failure, Error, InProgress, Skipped));

        private final State defaultState;
        private final List<State> orderedTerminalStates;

        ContainerStateComputationPolicy(final State defaultState, final List<State> orderedTerminalStates) {
            this.defaultState = defaultState;
            this.orderedTerminalStates = orderedTerminalStates;
        }

        public State computeContainerState(final Set<State> states) {
            for (final State expectedState : orderedTerminalStates) {
                if (states.contains(expectedState)) {
                    return expectedState;
                }
            }
            return defaultState;
        }
    }

    enum State {
        Aborted(true, false, false, "Aborted"),
        InProgress(false, false, false, "In progress"),
        Error(true, true, false, "Error"),
        Failure(true, true, true, "Failure"),
        Ok(true, true, true, "Success"),
        None(false, false, false, ""),
        Skipped(true, false, false, "Skipped");

        public final boolean isTerminal;
        public final boolean wasSent;
        public final boolean hasResponse;
        public final String displayName;

        State(final boolean isTerminal, final boolean wasSent, final boolean hasResponse, final String displayName) {
            this.isTerminal = isTerminal;
            this.wasSent = wasSent;
            this.hasResponse = hasResponse;
            this.displayName = displayName;
        }
    }

    State getState();

    T setState(State state);

    String getName();

    T setName(String name);

    String getFullName();

    T setFullName(String fullName);

    ContainerTestResult getParent();

    T setParent(ContainerTestResult parent);

    E getEntity();

    T setEntity(E entity);

    long getElapsedTime();

    T setElapsedTime(long elapsedTime);

    T computeNameAndFullName();

    T walk(Consumer<TestResult> visitor);

    boolean matches(EntityTo entity);
}
