package com.restlet.client.tests.impl;

import com.restlet.client.function.Function;
import com.restlet.client.model.EntityTo;
import com.restlet.client.model.impl.RetryConfigurationImpl;
import com.restlet.client.net.http.HttpClientResultTo;
import com.restlet.client.net.request.RetryConfiguration;
import com.restlet.client.tests.SingleRequestTestResult;
import com.restlet.client.tests.SingleRequestTryTestResult;
import com.restlet.client.tests.TestResult.State;
import com.restlet.client.tests.asserts.AssertionResultTo;
import com.restlet.client.utils.FunctionalUtils;
import com.restlet.client.utils.Objects;
import com.restlet.client.utils.Sequence;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import static com.restlet.client.tests.TestResult.ContainerStateComputationPolicy.AT_LEAST_ONE_SUCCESS;
import static com.restlet.client.tests.TestResult.State.Aborted;
import static com.restlet.client.tests.TestResult.State.InProgress;
import static com.restlet.client.tests.TestResult.State.Ok;
import static com.restlet.client.tests.impl.TestRunner.MESSAGE_TEMPLATE_REQUEST_UNSUCCESSFUL_AFTER_N_TRIES;
import static com.restlet.client.utils.FunctionalUtils.ifNotNull;
import static com.restlet.client.utils.StringUtils.format;
import static java.util.Arrays.asList;

public class SingleRequestTryTestResultImpl implements SingleRequestTryTestResult {

    private String message;
    private HttpClientResultTo httpResult;
    private final List<AssertionResultTo> assertionsResults = new ArrayList<>();
    private Throwable throwable;
    private State state = InProgress;
    private SingleRequestTestResult parent;
    private int displayIndex;

    @Override
    public String getMessage() {
        return message;
    }

    @Override
    public SingleRequestTryTestResultImpl setMessage(final String message) {
        this.message = message;
        return this;
    }

    @Override
    public HttpClientResultTo getHttpResult() {
        return httpResult;
    }

    @Override
    public SingleRequestTryTestResultImpl setHttpResult(final HttpClientResultTo httpResult) {
        this.httpResult = httpResult;
        return this;
    }

    @Override
    public List<AssertionResultTo> getAssertionsResults() {
        return assertionsResults;
    }

    @Override
    public SingleRequestTryTestResult addAssertionResult(final AssertionResultTo assertionResult) {
        this.assertionsResults.add(assertionResult);
        return this;
    }

    @Override
    public SingleRequestTryTestResult addAssertionResults(final List<AssertionResultTo> assertionResults) {
        this.assertionsResults.addAll(assertionResults);
        return this;
    }

    @Override
    public Throwable getThrowable() {
        return throwable;
    }

    @Override
    public SingleRequestTryTestResultImpl setThrowable(final Throwable throwable) {
        this.throwable = throwable;
        return this;
    }

    @Override
    public State getState() {
        return state;
    }

    @Override
    public SingleRequestTryTestResultImpl setState(final State state) {
        this.state = state;
        return this;
    }

    @Override
    public SingleRequestTestResult getParent() {
        return parent;
    }

    @Override
    public SingleRequestTryTestResultImpl setParent(final SingleRequestTestResult parent) {
        this.parent = parent;
        return this;
    }

    @Override
    public SingleRequestTryTestResult updateParentIfRequestComplete(final boolean isLastTry) {
        if (!isLastTry && !asList(Ok, Aborted).contains(getState())) {
            return this;
        }

        final RetryConfiguration retryConfiguration = ifNotNull(parent.getEntity().getRetryConfiguration(), FunctionalUtils.<RetryConfiguration>identity(), new RetryConfigurationImpl());
        if (retryConfiguration.isEnabled() && retryConfiguration.getRetryCount() > 1) {
            parent.setMessage(format(MESSAGE_TEMPLATE_REQUEST_UNSUCCESSFUL_AFTER_N_TRIES, retryConfiguration.getRetryCount()));
        } else {
            parent.setMessage(this.message);
        }

        final Set<State> states = Sequence.of(parent.getTriesResults())
                .map(new Function<SingleRequestTryTestResult, State>() {
                    @Override
                    public State apply(final SingleRequestTryTestResult tryResult) {
                        return tryResult.getState();
                    }
                })
                .toSet();

        parent.setState(AT_LEAST_ONE_SUCCESS.computeContainerState(states));
        return this;
    }

    @Override
    public int getDisplayIndex() {
        return displayIndex;
    }

    @Override
    public SingleRequestTryTestResult setDisplayIndex(final int displayIndex) {
        this.displayIndex = displayIndex;
        return this;
    }

    @Override
    public boolean matches(final EntityTo httpRequest) {
        return httpRequest != null && Objects.equals(getParent().getEntity().getId(), httpRequest.getId());
    }
}
