package com.restlet.client.utils;

import com.restlet.client.net.http.request.HttpRequestTo;
import com.restlet.client.net.uri.UriTo;

import java.util.ArrayList;
import java.util.List;

import static com.restlet.client.utils.StringUtils.oneNotBlank;

public class HttpUtils {

    public static List<String> validateUriAndMethod(final HttpRequestTo request) {
        final List<String> errors = new ArrayList<>();

        if (request.getMethod() == null) {
            errors.add("Method is not defined.");
        }
        if (request.getUri() == null || !isValid(request.getUri())) {
            errors.add("Missing URI.");
        }

        return errors;
    }

    public static boolean isValid(final UriTo uri) {
        return uri != null && oneNotBlank(uri.getHost(), uri.getPath());
    }

    public static boolean isSuccess(final int statusCode) {
        return isInRange(statusCode, 200, 299);
    }

    public static boolean isClientError(final int statusCode) {
        return isInRange(statusCode, 400, 499);
    }

    public static boolean isServerError(final int statusCode) {
        return isInRange(statusCode, 500, 599);
    }

    // Indicates that the server has not been reached
    public static boolean isConnectionError(final int statusCode) { return statusCode == 0; }

    private static boolean isInRange(final int valueToCheck, final int minValue, final int maxValue) {
        return valueToCheck >= minValue && valueToCheck <= maxValue;
    }

}
