package com.restlet.client.utils;

import com.restlet.client.platform.blob.Blob;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

public class Objects {

    public static <T> boolean isNullOrEmpty(final T[] objects) {
        return objects == null || objects.length == 0;
    }

    public static <T> boolean isNullOrEmpty(final Collection<T> object) {
        return object == null || object.isEmpty();
    }

    public static <T, E> boolean isNullOrEmpty(final Map<T, E> object) {
        return object == null || object.isEmpty();
    }

    public static boolean isNullOrEmpty(final Blob blob) {
        return blob == null || blob.isEmpty();
    }

    public static boolean isNullOrEmpty(final String stringParameter) {
        return stringParameter == null || stringParameter.isEmpty();
    }

    @SafeVarargs
    public static <T> T firstNonNull(final T... values) {
        for (final T value : values) {
            if (value != null) {
                return value;
            }
        }

        throw new IllegalArgumentException("Expected at least one non null parameter");
    }

    public static <T> T checkNotNull(final T object) {
        return checkNotNull(object, "Expected a non-null parameter.");
    }

    public static <T> T checkNotNull(final T object, final String message) {
        if (object == null) {
            throw new IllegalArgumentException(message);
        }
        return object;
    }

    public static String firstNonBlank(final String... values) {
        for (final String value : values) {
            if (!StringUtils.isBlank(value)) {
                return value;
            }
        }

        throw new IllegalArgumentException("Expected at least one non blank parameter");
    }

    public static <T> T requireNonNull(final T obj) {
        if (obj == null)
            throw new NullPointerException();
        return obj;
    }

    public static boolean equals(final Object a, final Object b) {
        return (a == b) || (a != null && a.equals(b));
    }

    public static int hashCode(final Object o) {
        return o != null ? o.hashCode() : 0;
    }

    public static <T> boolean contains(final T[] elements, final T element) {
        return Arrays.asList(elements).contains(element);
    }

    public static boolean areNotNull(final Object... objects) {
        for (final Object object : objects) {
            if (object == null) {
                return false;
            }
        }
        return true;
    }

}
